@@ -84,7 +84,8 @@ int kadm5_create(kadm5_config_params *params)
      */
     if ((retval = kadm5_get_config_params(context, 1,
                                           params, &lparams))) {
-        com_err(progname, retval, "while looking up the Kerberos configuration");
+        com_err(progname, retval, _("while looking up the Kerberos "
+                                    "configuration"));
         return 1;
     }
 
@@ -107,10 +108,11 @@ int kadm5_create_magic_princs(kadm5_config_params *params,
         return retval;
     if ((retval = kadm5_init(context, progname, NULL, NULL, params,
                              KADM5_STRUCT_VERSION,
-                             KADM5_API_VERSION_3,
+                             KADM5_API_VERSION_4,
                              db5util_db_args,
                              &handle))) {
-        com_err(progname, retval, "while initializing the Kerberos admin interface");
+        com_err(progname, retval, _("while initializing the Kerberos admin "
+                                    "interface"));
         return retval;
     }
 
@@ -123,31 +125,6 @@ int kadm5_create_magic_princs(kadm5_config_params *params,
     return retval;
 }
 
-/*
- * Function: build_name_with_realm
- *
- * Purpose: concatenate a name and a realm to form a krb5 name
- *
- * Arguments:
- *
- *      name    (input) the name
- *      realm   (input) the realm
- *
- * Returns:
- *
- *      pointer to name@realm, in allocated memory, or NULL if it
- *      cannot be allocated
- *
- * Requires: both strings are null-terminated
- */
-static char *build_name_with_realm(char *name, char *realm)
-{
-    char *n;
-
-    asprintf(&n, "%s@%s", name, realm);
-    return n;
-}
-
 /*
  * Function: add_admin_princs
  *
@@ -182,7 +159,7 @@ static int add_admin_princs(void *handle, krb5_context context, char *realm)
         goto clean_and_exit;
     }
     memset(&ai_hints, 0, sizeof(ai_hints));
-    ai_hints.ai_flags = AI_CANONNAME;
+    ai_hints.ai_flags = AI_CANONNAME | AI_ADDRCONFIG;
     gai_error = getaddrinfo(localname, (char *)NULL, &ai_hints, &ai);
     if (gai_error) {
         ret = EINVAL;
@@ -192,9 +169,8 @@ static int add_admin_princs(void *handle, krb5_context context, char *realm)
     }
     if (ai->ai_canonname == NULL) {
         ret = EINVAL;
-        fprintf(stderr,
-                "getaddrinfo(%s): Cannot determine canonical hostname.\n",
-                localname);
+        fprintf(stderr, _("getaddrinfo(%s): Cannot determine canonical "
+                          "hostname.\n"), localname);
         freeaddrinfo(ai);
         goto clean_and_exit;
     }
@@ -214,7 +190,7 @@ static int add_admin_princs(void *handle, krb5_context context, char *realm)
     }
     if (asprintf(&service_name, "kadmin/%s", ai->ai_canonname) < 0) {
         ret = ENOMEM;
-        fprintf(stderr, "Out of memory\n");
+        fprintf(stderr, _("Out of memory\n"));
         freeaddrinfo(ai);
         goto clean_and_exit;
     }
@@ -283,10 +259,13 @@ int add_admin_princ(void *handle, krb5_context context,
 
     memset(&ent, 0, sizeof(ent));
 
-    fullname = build_name_with_realm(name, realm);
+    if (asprintf(&fullname, "%s@%s", name, realm) < 0) {
+        com_err(progname, ENOMEM, _("while appending realm to principal"));
+        return ERR;
+    }
     ret = krb5_parse_name(context, fullname, &ent.principal);
     if (ret) {
-        com_err(progname, ret, "while parsing admin principal name");
+        com_err(progname, ret, _("while parsing admin principal name"));
         return(ERR);
     }
     ent.max_life = lifetime;
@@ -298,7 +277,7 @@ int add_admin_princ(void *handle, krb5_context context,
                                  "to-be-random");
     if (ret) {
         if (ret != KADM5_DUP) {
-            com_err(progname, ret, "while creating principal %s", fullname);
+            com_err(progname, ret, _("while creating principal %s"), fullname);
             krb5_free_principal(context, ent.principal);
             free(fullname);
             return ERR;
@@ -307,7 +286,8 @@ int add_admin_princ(void *handle, krb5_context context,
         /* only randomize key if we created the principal */
         ret = kadm5_randkey_principal(handle, ent.principal, NULL, NULL);
         if (ret) {
-            com_err(progname, ret, "while randomizing principal %s", fullname);
+            com_err(progname, ret, _("while randomizing principal %s"),
+                    fullname);
             krb5_free_principal(context, ent.principal);
             free(fullname);
             return ERR;
@@ -316,7 +296,8 @@ int add_admin_princ(void *handle, krb5_context context,
         ent.attributes = attrs;
         ret = kadm5_modify_principal(handle, &ent, KADM5_ATTRIBUTES);
         if (ret) {
-            com_err(progname, ret, "while setting attributes on %s", fullname);
+            com_err(progname, ret, _("while setting attributes on %s"),
+                    fullname);
             krb5_free_principal(context, ent.principal);
             free(fullname);
             return ERR;
