@@ -314,8 +314,9 @@ process_chpw_request(krb5_context context, void *server_handle, char *realm,
 
 chpwfail:
 
-    clear.length = 2 + strlen(strresult);
-    clear.data = (char *) malloc(clear.length);
+    ret = alloc_data(&clear, 2 + strlen(strresult));
+    if (ret)
+        goto bailout;
 
     ptr = clear.data;
 
